(function () {
    'use strict';

    angular
        .module('app.core')
        .config(keyEventsConfig);

    /** @ngInject */
    function keyEventsConfig(keyEventsProvider) {
        // Each shortcut() operator gets called with the event object and the
        // "is" object. The goal here is to add ".is" properties based on the
        // state of the event.
        keyEventsProvider.addShortcut(
            function operator(event, is) {
                // The ESC key is more consistently reported in the keydown
                // event (as opposed to the keypress event). As such, we'll
                // consider it false unless the event type is appropriate.
                is.esc = ( ( event.type === 'keydown' ) && ( event.which === 27 ) );

                // The CMD-ENTER combination is more consistently reported
                // in the keydown event (as opposed to the keypress event). As
                // such, we'll consider it false unless the event type is appropriate.
                is.cmdEnter = ( ( event.type === 'keydown' ) && ( event.which === 13 ) && event.metaKey );
                is.input = (
                    ( event.target.tagName === 'BUTTON' ) ||
                    ( event.target.tagName === 'INPUT' ) ||
                    ( event.target.tagName === 'SELECT' ) ||
                    ( event.target.tagName === 'TEXTAREA' )
                );
            }
        );
    }

})();
