(function () {
    'use strict';

    angular
        .module('app.core')
        .directive('carouselCover', carouselCoverDirective);

    /** @ngInject */
    function carouselCoverDirective(keyEvents, STA, config, $timeout, $window) {
        return {
            restrict: 'E',
            templateUrl: 'app/core/directives/carousel/carousel.html',
            scope: {
                items: '=',
                auto: '=',
                arrows: '='
            },
            link: function (scope, element, attrs) {
                var keyHandler;
                var carouselList = $(element).find('.carousel-list');

                scope.onFocusElement = onFocusElement;
                scope.onBlurElement = onBlurElement;
                scope.getLeft = getLeft;
                scope.getRight = getRight;
                scope.checkLeft = false;
                scope.checkRight = false;

                scope.carousel = {
                    max: 30,
                    initial: 0,
                    index: 0,
                    item: null,
                    count: 0
                };

                //// ---------

                var watchItems = scope.$watch('items', function (values) {
                    if (values) {
                        var timeoutItems = $timeout(function () {
                            scope.carousel.index = scope.carousel.initial;

                            if (scope.auto) {
                                if (config('carouselIndex') || config('carouselIndex') === 0) {
                                    scope.carousel.index = config('carouselIndex');
                                }

                                selectItem(scope.carousel.index);
                            }

                            $timeout.cancel(timeoutItems);
                        });
                    }
                });

                var watchCarouselIndex = scope.$watch('carousel.index', function () {
                    checkArrow();
                }, true);


                function checkArrow() {
                    if (scope.totalItems.length <= 2) {
                        scope.checkRight = false;
                        scope.checkLeft = false;
                        return false;
                    }

                    scope.checkRight = scope.carousel.index < (scope.totalItems.length - 1);
                    scope.checkLeft = scope.carousel.index > 0;
                }


                function getLeft($event) {
                    preventEvent($event);
                    scope.carousel.index--;

                    if (scope.carousel.index < 0) {
                        scope.carousel.index = 0;
                    }

                    selectItem(scope.carousel.index);
                }


                function getRight($event) {
                    preventEvent($event);
                    scope.carousel.index++;

                    var count = Math.min(scope.totalItems.length, scope.carousel.max);
                    if (scope.carousel.index >= count) {
                        scope.carousel.index = count - 1;
                    }

                    selectItem(scope.carousel.index);
                }


                function selectItem(value) {
                    scope.carousel.item = scope.items[value];
                    var item = $(element).find('#item-' + value);

                    if (!_.isEmpty(item)) {
                        item[0].focus();

                        config('carouselIndex', value);
                        var translation = -485;
                        if ($window.outerHeight === 1080) {
                            translation = -755;
                        }

                        translate(value * translation);
                    }
                }


                function translate(value) {
                    carouselList
                        .css('-webkit-transform', 'translate(' + value + 'px, 0px)')
                        .css('-moz-transform', 'translate(' + value + 'px, 0px)')
                        .css('-ms-transform', 'translate(' + value + 'px, 0px)')
                        .css('-o-transform', 'translate(' + value + 'px, 0px)')
                        .css('transform', 'translate(' + value + 'px, 0px)')
                }


                var focusin = function ($event) {
                    preventEvent($event);

                    onFocusElement($event)
                    scope.carousel.index = scope.carousel.index || scope.carousel.initial;
                    selectItem(scope.carousel.index);
                };
                carouselList.on('focusin', focusin);


                var keydown = function ($event) {
                    switch ($event.keyCode) {
                        case STA.keys.VK_RIGHT:
                            if (scope.carousel.index >= (scope.totalItems.length - 1)) {
                                preventEvent($event);
                            }
                            break;

                        case STA.keys.VK_LEFT:
                            if (scope.carousel.index === 0) {
                                preventEvent($event);
                            }
                            break;
                    }
                };
                carouselList.on('keydown', keydown);


                var focusout = function ($event) {
                    preventEvent($event);
                    onBlurElement($event);
                };
                carouselList.on('focusout', focusout);


                function onFocusElement($event) {
                    preventEvent($event);

                    // Key handler
                    if (!keyHandler) {
                        keyHandler = keyEvents
                            .handler(1)
                            .keyup(function handleKeyEvent(event) {
                                preventEvent(event);

                                switch (event.keyCode) {
                                    case STA.keys.VK_RIGHT:
                                        getRight(event);
                                        break;

                                    case STA.keys.VK_LEFT:
                                        getLeft(event);
                                        break;
                                }

                                scope.$apply();
                            });
                    }
                }


                function onBlurElement($event) {
                    preventEvent($event);

                    if (keyHandler) {
                        keyHandler.teardownType('keyup');
                        keyHandler = null;
                    }
                }


                function preventEvent($event) {
                    if ($event) {
                        $event.preventDefault();
                        $event.stopPropagation();
                    }
                }


                scope.$on('$destroy', function () {
                    onBlurElement();
                    carouselList.off('focusout', focusout);
                    carouselList.off('focusin', focusin);
                    carouselList.off('keydown', keydown);

                    if (!scope.auto) {
                        scope.carousel.index = scope.carousel.initial;
                        config('carouselIndex', scope.carousel.index);
                    }

                    watchItems();
                    watchCarouselIndex();
                });
            }
        };
    }
})();
