(function () {
    'use strict';

    angular
        .module('app.core')
        .directive('keyboardInline', keyboardInline);

    /** @ngInject */
    function keyboardInline(keyEvents, STA) {
        return {
            restrict: 'EA',
            templateUrl: 'app/core/directives/keyboard/keyboardInline.html',
            scope: {
                modelInput: '=model'
            },
            link: function (scope, elem, attrs) {
                var keyHandler;
                scope.keyboard = [
                    [' ', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'],
                    ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'],
                    [',', '.', '_', '-', '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '+', '=', '[', ']', '{', '}', ';', ':', '?']
                ];
                scope.keyboardStates = ['#123', '.,-', 'abc'];

                scope.activeKeyboard = 0;
                scope.upperText = 0;
                scope.selectItem = selectItem;
                scope.deleteItem = deleteItem;
                scope.changeState = changeState;
                scope.getKeyboard = getKeyboard;
                scope.changeUppertext = changeUppertext;

                keyHandler = keyEvents.handler(1).keyup(function handleKeyEvent(event) {
                    switch (event.keyCode) {
                        case STA.keys.VK_BLUE:
                            deleteItem(event);
                            scope.$apply();
                            break;
                    }
                });

                scope.$on('$destroy', function () {
                    if (keyHandler) {
                        keyHandler.teardownType('keyup');
                    }
                });

                function changeUppertext($event) {
                    preventEvent($event)
                    scope.upperText = !scope.upperText;
                }

                function getKeyboard() {
                    return scope.keyboard[scope.activeKeyboard];
                }

                function changeState($event) {
                    preventEvent($event)
                    scope.activeKeyboard = (scope.activeKeyboard + 1) % scope.keyboardStates.length;
                }

                function selectItem($event, index) {
                    preventEvent($event)
                    var letter = (scope.upperText) ? String(scope.keyboard[scope.activeKeyboard][index]).toUpperCase() : scope.keyboard[scope.activeKeyboard][index];
                    scope.modelInput = scope.modelInput.concat(letter);
                }

                function deleteItem($event) {
                    preventEvent($event)
                    scope.modelInput = scope.modelInput.slice(0, -1);
                }

                function preventEvent($event) {
                    if ($event) {
                        $event.preventDefault();
                        $event.stopPropagation();
                    }
                }
            }
        };
    }
}());
