(function () {
    'use strict';

    angular
        .module('app.core')
        .directive('searchInput', searchInput)
        .filter('search', searchFilter);

    /** @ngInject */
    function searchInput(keyEvents, STA) {
        return {
            restrict: 'EA',
            templateUrl: 'app/core/directives/search/search.html',
            scope: {
                searchTitle: '='
            },
            link: function (scope, elem, attrs) {
                scope.enableKeyboard = false;
                scope.clickEnableKeyboard = clickEnableKeyboard;
                scope.item = {
                    search: ''
                };

                var watchSearch = scope.$watch('item.search', function (item) {
                    scope.searchTitle = item;
                });

                var keyHandler = keyEvents.handler(1).keyup(function handleKeyEvent(event) {
                    switch (event.keyCode) {
                        case STA.keys.VK_BLUE:
                            deleteItem();
                            break;
                    }
                });

                scope.$on('$destroy', function () {
                    if (keyHandler) {
                        keyHandler.teardownType('keyup');
                    }

                    watchSearch();
                });

                function deleteItem(evt) {
                    evt.preventDefault();
                    evt.stopPropagation();
                    scope.item.search = scope.item.search.slice(0, -1);
                }

                function clickEnableKeyboard(evt) {
                    evt.preventDefault();
                    evt.stopPropagation();
                    scope.enableKeyboard = !scope.enableKeyboard;
                }
            }
        };
    }

    /** @ngInject */
    function searchFilter() {
        function escapeRegExp(string) {
            return string.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1");
        }

        return function (items, searchText) {
            var filtered = [];

            if (!searchText) {
                return items;
            }

            var regex = new RegExp('\\b' + escapeRegExp(searchText), 'i');
            angular.forEach(items, function (item) {
                if (regex.test(item.name)) {
                    filtered.push(item);
                } else if (regex.test(item.description)) {
                    filtered.push(item);
                }
            });

            return filtered;
        };
    }
}());
