(function () {
    'use strict';

    angular
        .module('app.core')
        .directive('videoPlayer', videoPlayerDirective);

    /** @ngInject */
    function videoPlayerDirective(STA, UI, Videoplayer, keyEvents, $location) {
        return {
            restrict: 'AE',
            templateUrl: 'app/core/directives/videoPlayer/videoPlayer.html',
            scope: {
                data: '=videoData'
            },
            link: function (scope, element, attrs) {
                UI.hideSplash();
                var playerVideo;

                // Setup video controls
                scope.showInfo = true;
                scope.showControls = true;
                scope.showMultimedia = false;
                scope.progress = 0;
                scope.bufferProgress = 0;
                scope.multimediaItems = [];

                scope.play = play;
                scope.pause = pause;
                scope.forward = forward;
                scope.rewind = rewind;
                scope.hidePlayer = hidePlayer;
                scope.subtitle = subtitle;
                scope.audio = audio;
                scope.mouseclick = mouseclick;
                scope.mouseover = mouseover;
                scope.mouseout = mouseout;
                scope.selectMultimedia = selectMultimedia;
                scope.getLabel = getLabel;

                var timeoutPlaycontrol;
                var timeoutInfo;
                var timeInfo = 3000;
                var selectedSubtitle = false;
                var selectedMultiaudio = false;
                var videoSpeed = 1;

                var watchData = scope.$watch('data', function (data) {
                    if (data && data.url) {
                        init();
                    }
                });

                // Key handler
                var keyHandler = keyEvents
                    .handler(1)
                    .keyup(function handleKeyEvent(event) {
                            clearAll();

                            switch (event.keyCode) {
                                case STA.keys.VK_ENTER:
                                case STA.keys.VK_UP:
                                case STA.keys.VK_DOWN:
                                case STA.keys.VK_RIGHT:
                                case STA.keys.VK_LEFT:
                                    //if (!scope.showControls || !scope.showInfo) {
                                        showAll();
                                        clearPlayer();
                                    //}
                                    break;

                                case STA.keys.VK_PLAY:
                                    play();
                                    break;

                                case STA.keys.VK_PAUSE:
                                    pause();
                                    break;

                                case STA.keys.VK_STOP:
                                    stop();
                                    break;

                                case STA.keys.VK_REWIND:
                                    rewind();
                                    break;

                                case STA.keys.VK_FAST_FWD:
                                    forward();
                                    break;
                                case STA.keys.VK_PREV:
                                    fRewind();
                                    break;
                                case STA.keys.VK_NEXT:
                                    fForward();
                                    break;
                                default :
                                    showAll();
                                    clearPlayer();
                                    break;
                            }
                        }
                    );

                scope.$on('$destroy', function () {
                    if (keyHandler) {
                        keyHandler.teardownType('keyup');
                    }

                    watchData();
                });

                function mouseclick($event) {
                    preventEvent($event)
                    showAll();
                    clearPlayer();
                }

                function mouseover($event) {
                    preventEvent($event)
                    showAll();
                    clearAll();
                }

                function mouseout($event) {
                    preventEvent($event)
                    clearPlayer();
                }

                function clearPlayer() {
                    setTimeOutInfo(timeInfo);
                    setTimeOutControl(timeInfo);
                }

                function setTimeOutInfo(pcTimeout) {
                    timeoutInfo = setTimeout(clearInfo, pcTimeout);
                }

                function setTimeOutControl(pcTimeout) {
                    timeoutPlaycontrol = setTimeout(clearPlaycontrol, pcTimeout);
                }

                function showAll() {
                    showInfo();
                    showPlaycontrol();
                }

                function showInfo() {
                    scope.showInfo = true;
                }

                function showPlaycontrol() {
                    scope.showControls = true;
                }

                function clearAll() {
                    clearTimeout(timeoutPlaycontrol);
                    clearTimeout(timeoutInfo);
                }

                function clearInfo() {
                    scope.showInfo = false;
                }

                function clearPlaycontrol() {
                    scope.showControls = false;
                }


                function init() {
                    playerVideo = new Videoplayer({
                        url: scope.data.url,
                        containerId: 'player',
                        autoplay: true
                    });

                    playerVideo.init();

                    var duration = 0;
                    var progress = 0;

                    // Event when time is updated
                    var whenTimeupdate = function (mseconds) {
                        scope.position = parseTime(mseconds / 1000);
                        progress = mseconds / (duration / 100)
                        scope.progress = progress;
                        playerVideo.setPlaybackSpeed(videoSpeed);
                        scope.$apply();
                    };

                    // Event when buffering video
                    var whenBuffering = function (buffProgress) {
                        scope.bufferProgress = (buffProgress * 100) - progress;
                    };

                    // Event when get info
                    var whenInfo = function (info) {
                        // If duration is Infinity, hide the duration and progress bar.
                        // Could be because live streaming
                        if (!isFinite(info.duration)) {
                            duration = null;
                            scope.duration = null;
                        } else {
                            duration = info.duration;
                            scope.duration = parseTime(info.duration / 1000);
                        }

                        scope.multimediaAudioItems = playerVideo.getAudioTracks();
                        scope.multimediaTextItems = playerVideo.getTextTracks();
                        scope.$apply();
                    };

                    // Event when change state
                    var whenStateName = function (stateName) {
                        console.log('stateName', stateName);
                    };

                    // Event when is play
                    var whenPlay = function () {
                        scope.isPlaying = videoSpeed === 1;
                        clearPlayer();

                        if (scope.isPlaying) {
                            focus('pause');
                        } else {
                            focus('play');
                        }

                        scope.$apply();
                    };

                    // Event when is paused
                    var whenPause = function () {
                        scope.isPlaying = false;
                        playerVideo.setPlaybackSpeed(1);
                        clearPlayer();
                        focus('play');
                        scope.$apply();
                    };

                    // Event when is waiting to load the video
                    var whenWaiting = function ($event) {
                        UI.loading(true);
                    };

                    // Event when playing the video
                    var whenPlaying = function ($event) {
                        UI.loading(false);
                    };

                    // Exit from the player
                    var exitFn = function ($event) {
                        redirect(scope.data.id);
                    };

                    playerVideo.on('timeupdate', whenTimeupdate);
                    playerVideo.on('buffering', whenBuffering);
                    playerVideo.on('info', whenInfo);
                    playerVideo.on('statechange', whenStateName);
                    playerVideo.on('play', whenPlay);
                    playerVideo.on('pause', whenPause);
                    playerVideo.on('waiting', whenWaiting);
                    playerVideo.on('playing', whenPlaying);
                    playerVideo.on('ended', exitFn);
                    playerVideo.on('error', exitFn);

                    showAll();
                    focus('play');

                    scope.$on('$destroy', function () {
                        playerVideo.off('timeupdate', whenTimeupdate);
                        playerVideo.off('buffering', whenBuffering);
                        playerVideo.off('info', whenInfo);
                        playerVideo.off('statechange', whenStateName);
                        playerVideo.off('play', whenPlay);
                        playerVideo.off('pause', whenPause);
                        playerVideo.off('waiting', whenWaiting);
                        playerVideo.off('playing', whenPlaying);
                        playerVideo.off('ended', exitFn);
                        playerVideo.off('error', exitFn);
                        playerVideo.exit();
                    });
                }

                function play($event) {
                    preventEvent($event);
                    if (videoSpeed !== 1) {
                        videoSpeed = 1;
                        scope.isPlaying = true;
                        focus('pause');
                    } else {
                        playerVideo.play();
                    }
                }

                function pause($event) {
                    preventEvent($event);
                    videoSpeed = 1;
                    playerVideo.pause();
                }

                function stop($event) {
                    preventEvent($event);
                    videoSpeed = 1;
                    playerVideo.stop();
                }

                function fForward($event) {
                    preventEvent($event);
                    videoSpeed = 1;
                    playerVideo.stepForward(10000);
                }

                function fRewind($event) {
                    preventEvent($event);
                    videoSpeed = 1;
                    playerVideo.stepBackward(10000);
                }

                function rewind($event) {
                    preventEvent($event);
                    videoSpeed = 1;
                    playerVideo.stepBackward(10000);
                }

                function forward($event) {
                    preventEvent($event);
                    videoSpeed = 1;
                    playerVideo.stepForward(10000);
                }

                function selectAudio($event, id) {
                    preventEvent($event);
                    var ct = playerVideo.getCurrentTime();
                    playerVideo.stop();
                    var timeout1 = setTimeout(function () {
                        playerVideo.setAudioTrack(id);
                        clearTimeout(timeout1);
                    },0) ;
                    var timeout2 = setTimeout(function () {
                        playerVideo.play();
                        clearTimeout(timeout2);
                    },0) ;
                    var timeout3 = setTimeout(function () {
                        playerVideo.setCurrentTime(ct);
                        clearTimeout(timeout3);
                    },0) ;
                }

                function selectSubtitle($event, id) {
                    preventEvent($event);
                    playerVideo.setTextTrack(id);
                }

                function hidePlayer($event) {
                    preventEvent($event);
                    $location.path('detail/' + scope.data.id);
                }

                function selectMultimedia($event, id) {
                    preventEvent($event);
                    if (selectedSubtitle) {
                        selectSubtitle($event, id);
                    } else {
                        selectAudio($event, id);
                    }
                }

                function subtitle($event) {
                    preventEvent($event);
                    scope.multimediaItems = scope.multimediaTextItems;

                    if (scope.multimediaTextItems) {
                        scope.showMultimedia = (scope.showMultimedia === 'subtitle') ? scope.showMultimedia = false : scope.showMultimedia = 'subtitle';
                        selectedSubtitle = true;
                        selectedMultiaudio = false;
                    }
                }

                function audio($event) {
                    preventEvent($event);
                    scope.multimediaItems = scope.multimediaAudioItems;

                    if (scope.data.multiaudio) {
                        scope.showMultimedia = (scope.showMultimedia === 'audio') ? scope.showMultimedia = false : scope.showMultimedia = 'audio';
                        selectedSubtitle = false;
                        selectedMultiaudio = true;
                    }
                }

                function getLabel(label, index) {
                    if (label && label !== '') {
                        return label;
                    } else {
                        if (selectedSubtitle) {
                            return 'Subtitle: ' + index;
                        } else {
                            return 'Track: ' + index;
                        }
                    }
                }

                function parseTime(seconds) {
                    seconds = Math.round(parseFloat(seconds));
                    var s = seconds % 60;
                    var m = Math.floor(seconds / 60);
                    return pad(m) + ':' + pad(s);
                }

                function pad(s) {
                    s += '';
                    if (s.length < 2)
                        s = '0' + s;

                    return s;
                }

                function focus(id) {
                    var timeoutFocus = setTimeout(function () {
                        UI.focus(id, true);
                        clearTimeout(timeoutFocus);
                    });
                }

                function redirect(id) {
                    playerVideo.stop();
                    var timeoutLocation = setTimeout(function () {
                        window.location = './#/detail/' + id;
                        clearTimeout(timeoutLocation);
                        playerVideo.exit();
                    });
                }

                function preventEvent($event) {
                    if ($event) {
                        $event.preventDefault();
                        $event.stopPropagation();
                    }
		        }

                function getCurrentTime() {
                    return playerVideo.currentTime;
                }

                function setCurrentTime(ct) {
                    playerVideo.currentTime = ct;
                }
            }
        }
    }
})();
